

#include "properties.h"
#include "stringhash.h"
#include "stringtokenizer.h"
#include "search.h"


GHashTable *properties;


void parse_var_line (gchar* line, gint lineno)
{
	TokenizerPtr st = new_tokenizer (line, "\t");
	gchar *stvar = next_token(st);
	gchar *stvalue = remaining_text (st);
	gchar *var, *value;
#ifdef HDEBUG
	var = h_alloc (strlen (stvar) + 1, "var", "parse_var_line", 19);
#else
	var = g_malloc (strlen (stvar) + 1);
#endif

#ifdef HDEBUG
	value = h_alloc (strlen (stvalue) + 1, "value", "parse_var_line", 20);
#else
	value = g_malloc (strlen (stvalue) + 1);
#endif
	
	strcpy (var, stvar);
	strcpy (value, stvalue);
	
#ifdef HDEBUG
	h_free (stvar, "stvar", "parse_var_line", 25);
	h_free (stvalue, "stvalue", "parse_var_line", 26);
#else
	g_free (stvar);
	g_free (stvalue);
#endif
	free_tokenizer (st);
	
	g_hash_table_insert (properties, var, value);

	//printf ("%s=%s\n", var, value);
}

void parse_properties_line (gchar* line, gint lineno)
{
	TokenizerPtr st = new_tokenizer(line, "\t");
	gchar *tag = next_token(st);
	gchar *parseline = remaining_text(st);
	
	//printf ("tag: \"%s\", %d\n", tag, strlen(tag));
	//printf ("parse line...\n%s\n", parseline);

	if ((strcmp (VAR_TAG, tag) == 0) ||
		(strcmp (UVAR_TAG, tag) == 0) ||
		(strcmp (IVAR_TAG, tag) == 0))
	{
		//printf ("var tag found: %s\n", parseline);
		parse_var_line (parseline, lineno);
	}
	else if (strcmp (ENGINE_TAG, tag) == 0)
	{
		//printf ("engine tag found:\n\t%s\n", parseline);
		add_engine_from_config_line(parseline, lineno);
	}
	else
	{
		printf ("unknown tag: \"%s\", ignoring config [%d]\n", tag, lineno);
	}
	free_tokenizer (st);
#ifdef HDEBUG
	h_free (tag, "tag", "parse_properties_line", 49);
	h_free (parseline, "parseline", "parse_properties_line", 50);
#else
	g_free (tag);
	g_free (parseline);
#endif
}


/* Read in the properties files, sending each line to an
   appropriate handler. */
gint read_properties()
{
	FILE *f;
	gchar *line;
	gint linecounter = 0;
	
	f = fopen (".netferretrc", "r");
	if (!f) {
		f = fopen ("ferret.txt", "r");
		if (!f) {
			printf ("Net Ferret error: could not find search config file \".netferretrc\" or \"ferret.txt\". Aborting.\n");
			return FALSE;
		}
	}
	
	/* read ferret configuration file, passing each line to
	   parse_properties_line which begins the parsing process */
#ifdef HDEBUG
	line = h_alloc (10240, "line", "read_properties", 73);
#else
	line = g_malloc (10240);
#endif

	while ((fgets(line, 10240, f)) != NULL)
	{
		line[strlen(line)-1] = 0;
		linecounter ++;
		if (strlen(line) <= 0) continue;
		if (line[0] == '#') continue;
		parse_properties_line (line, linecounter);
		//printf ("finished parsing: \"%s\"\n", line);
		line[0] = 0;
	}
	
//	fclose(f);
#ifdef HDEBUG
	h_free (line, "line", "read_properties", 86);
#else
	g_free (line);
#endif

	//printf ("read properties ok.\n");
	
	return TRUE;
}

gint init_properties()
{
	gint ret;
	
	//printf ("init properties table.\n");
	//if (properties) g_free (properties);
	properties = g_hash_table_new ( (GHashFunc)string_hashcode,
									(GCompareFunc)string_compare );
	//printf ("init_search_config.\n");
	init_search_config();
	
	//printf ("read props.\n");
	ret = read_properties();

	return ret;
}


gchar *get_property (gchar *key) {
	gchar *ret = (gchar*)g_hash_table_lookup (properties, key);
	//printf ("resource requested: \"%s\"\n", key);
	if (ret == NULL)
		printf ("error: resource \"%s\" not found!\n", key);
//	else
//		printf ("resource found: \"%s\"\n", ret);
	return ret;
}

int get_int_property (gchar *key, gint dflt)
{
	char* sval = get_property (key);
	if (sval == NULL) return dflt;
	return atoi (sval);
}

/*	As implemented, this method will either complete successfully and exit,
	or it will encounter a file error and immediately exit. Net Ferret cannot
	run without access to a ".netferretrc" file.
*/
void install_default_properties() {
	gint err;
	FILE *f = fopen (".netferretrc", "w");
		
	if (f) {
		err = fputs (DEFAULT_PROPERTIES, f);
	}
	else {
		printf ("Could not open props file for writing.\nMost likely cause: insufficient access\nprivlidges in the current directory.\nTry \"cd\" to you home directory, then\nre-run \"ferret_applet\". Exiting.\n");
		exit(0);
	}
	if (err == EOF) {
		printf ("Error writing default properties file. Error [%d] Exiting.\n",
				err);
		exit(0);
	}
	err = fflush(f);
	if (err) {
		printf ("Error flushing default properties file. Error [%d] Exiting.\n",
				err);
		exit(0);
	}
	err = fclose(f);
	if (err) {
		printf ("Error closing default properties file. Error [%d] Exiting.\n",
				err);
		exit(0);
	}	
}


void dispose_property_cb (gpointer key,
						  gpointer value,
						  gpointer user_data)
{
#ifdef HDEBUG
	h_free (key, "key", "dispose_property_cb", 167);
	h_free (value, "value", "dispose_property_cb", 168);
#else
	g_free (key);
	g_free (value);
#endif
}


void dispose_properties()
{
	/* dispose each entry */
	g_hash_table_foreach (properties,
		(GHFunc)dispose_property_cb, NULL);

	g_hash_table_destroy(properties);		
}

